@@ -206,7 +206,8 @@ class object(univention.admin.handlers.simpleComputer, nagios.Support):
 		self.nagios_ldap_pre_create()
 
 	def _ldap_addlist(self):
-		return [('objectClass', [b'top', b'univentionHost', b'univentionClient', b'person'])]
+		al = super(object, self)._ldap_addlist()
+		return al + [('objectClass', [b'top', b'univentionHost', b'univentionClient', b'person'])]
 
 	def _ldap_post_create(self):
 		univention.admin.handlers.simpleComputer._ldap_post_create(self)
